// HyperMD, copyright (c) by laobubu
// Distributed under an MIT license: http://laobubu.net/HyperMD/LICENSE
//
// POWERPACK for "addon/fold-code"
//
// This module provides `MermaidRenderer` for FoldCode addon
// so that you can render flowchart / diagram with powerful [mermaid](https://mermaidjs.github.io/)
//
// By default the renderer is enabled. You may disable it by setting `hmdFoldCode.mermaid` to `false`
//
// **Example**: https://laobubu.net/HyperMD/docs/examples/mermaid.html
//
// :hint: to change mermaid configuration
define(["require", "exports", "mermaid", "codemirror", "../addon/fold-code", "../addon/fold"], function (require, exports, _mermaid_module, CodeMirror, fold_code_1, fold_1) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    exports.MermaidRenderer = void 0;
    /** mermaid */
    var mermaid = _mermaid_module || this['mermaid'] || window['mermaid'];
    var MermaidRenderer = function (code, info) {
        var el = document.createElement('div');
        el.setAttribute('class', 'hmd-fold-code-image hmd-fold-code-mermaid');
        el.innerHTML = code;
        mermaid.run({ nodes: [el] });
        return el;
    };
    exports.MermaidRenderer = MermaidRenderer;
    if (typeof mermaid === "object") {
        CodeMirror.defineOption("mermaid", null, function (cm) {
            (0, fold_code_1.getAddon)(cm).clear("mermaid");
            (0, fold_1.getAddon)(cm).startFold();
        });
        (0, fold_code_1.registerRenderer)({
            name: "mermaid",
            pattern: /^mermaid$/i,
            renderer: exports.MermaidRenderer,
            suggested: true,
        });
    }
    else {
        console.error("[HyperMD] PowerPack fold-code-with-mermaid loaded, but mermaid not found.");
    }
});
